
from seti_python.class_tools import BasicResponse

class Commissions:
    fixed:int
    percentage:int
    def  __init__(self,data):
        self.fixed=data["fixed"]
        self.percentage=data["percentage"]
        
class Taxes:
    ICA: 0
    FTE: 0
    IVA: 0
    def  __init__(self,data):
        self.ICA=data["ICA"]
        self.FTE=data["FTE"]
        self.IVA=data["IVA"]
    
        

# Read Payment
class ResponseReadPaymentData:
    commission :str
    currency:str
    description:str
    payer_email:str
    payer_name:str
    payment_id:str
    payment_method_type:str
    payment_status:str
    redirect_url:str
    reference:str
    timestamp:dict
    url :str
    value :int
    taxes : Taxes
    commission:Commissions
    def __init__(self,data:dict):
        try:
            self.commission =data['commission']
            self.currency= data["currency"]
            self.description= data["description"]
            self.payer_email=data["payerEmail"]
            self.payer_name= data["payerName"]
            self.payment_id= data["paymentId"]
            self.payment_method_type= data["paymentMethodType"]
            self.payment_status= data["paymentStatus"]
            self.redirect_url=data["redirectUrl"]
            self.reference= data["reference"]
            self.timestamp= data["timestamp"]
            self.url = data["url"]
            self.value = data["value"]
            self.taxes = Taxes(data['taxes'])
            self.commission= Commissions(data['commission'])
        except Exception:
            return
                
class ResponseReadPayment(BasicResponse):
    data: ResponseReadPaymentData
    def __init__(self,data:dict):
        self.msg=data['msg']
        self.res=data['res']
        if data['res'] == 200:
            self.data = ResponseReadPaymentData(data['data'])
        else:
            self.data = ResponseReadPaymentData({})

# Read Cash

class ResponseReadCashData:
    balance:int
    currency:str
    currency_name:str
    def __init__(self,data:dict):
        self.balance = data['balance']
        self.currency =  data['currency']
        self.currency_name = data['currencyName']
        
class ResponseReadCash(BasicResponse):
    data: ResponseReadPaymentData
    def __init__(self,data:dict):
        self.msg=data['msg']
        self.res=data['res']
        if data['res'] == 200:
            self.data = ResponseReadPaymentData(data['data'])
        else:
            self.data = ResponseReadPaymentData({})
        
# Read total CashOUts

class ResponseReadCashOutData:
    total_cashout:int
    currency:str
    currency_name:str
    def __init__(self,data:dict):
        self.total_cashout = data['totalCashout']
        self.currency =  data['currency']
        self.currency_name = data['currencyName']
        
class ResponseReadCashOut(BasicResponse):
    data: ResponseReadCashOutData
    def __init__(self,data:dict):
        self.msg=data['msg']
        self.res=data['res']
        if data['res'] == 200:
            self.data = ResponseReadCashOutData(data['data'])
        else:
            self.data = ResponseReadPaymentData({})

# Read transactions

class Transaction:
    currency_id:str
    from_nusiness_id:str
    cash_out_id:str
    value:int
    to_wallet_id:str
    from_wallet_id:str
    timestamp:dict
    to_business_id:str
    payment_id:str
    transaction_iype:str
    id:str
    def  __init__(self,data):
        self.currency_id=data["currencyId"]
        self.cash_out_id=data["cashOutId"]
        self.value=data["value"]
        self.to_wallet_id=data["toWalletId"]
        self.from_wallet_id=data["fromWalletId"]
        self.timestamp=data["timestamp"]
        self.payment_id=data["paymentId"]
        self.transaction_type=data["transactionType"]
        if "toBusinessId" in data:
            self.to_business_id=data["toBusinessId"]
            self.from_nusiness_id=data["fromBusinessId"]
        if "id" in data:
            self.id=data["id"]
class ResponseReadTransactionsData:
    def __init__(self,data:dict):
        self.incoming = list(map(Transaction,data['incoming']))
        self.outgoing =  list(map(Transaction,data['outgoing']))
        
class ResponseReadTransactions(BasicResponse):
    data: ResponseReadTransactionsData
    def __init__(self,data:dict):
        self.msg=data['msg']
        self.res=data['res']
        if data['res'] == 200:
            self.data = ResponseReadTransactionsData(data['data'])
        else:
            self.data = ResponseReadPaymentData({})

# Read Wallet

class WalletData:
    balance:int
    currency:str
    locked:bool
    timestamp:dict
    wallet_id:str
    def  __init__(self,data):
        self.balance=data["balance"]
        self.currency=data["currency"]
        self.locked=data["locked"]
        self.timestamp=data["timestamp"]
        self.wallet_id =data["walletId"]
  

class ResponseReadWallet(BasicResponse):
    data: WalletData
    def __init__(self,data:dict):
        self.msg=data['msg']
        self.res=data['res']
        if data['res'] == 200:
            self.data = WalletData(data['data'])
        else:
            self.data = ResponseReadPaymentData({})
   
   
# Read Wallet Movements

  
class ResponseReadWallesMovemtsData:
    def __init__(self,data:dict):
        self.input = list(map(Transaction,data['input']))
        self.output =  list(map(Transaction,data['output']))
        
class ResponseReadWalletsTransactions(BasicResponse):
    data: ResponseReadWallesMovemtsData
    def __init__(self,data:dict):
        self.msg=data['msg']
        self.res=data['res']
        if data['res'] == 200:
            self.data = ResponseReadWallesMovemtsData(data['data'])
   
   


from seti_python.class_read import (
    ResponseReadCash,
    ResponseReadCashOut,
    ResponseReadPayment,
    ResponseReadTransactions,
    ResponseReadWallet,
    ResponseReadWalletsTransactions,
)
from seti_python.class_users import ResponseCreateUser, ResponseValidateUsersExist
from seti_python.wallet_class import ResponseCreateWallet
from seti_python.class_tools import (
    BasicResponse,
    CompleteBasicaResponse,
    ResponseCreateDispersions,
    ResponseCreateTransaction,
)
from seti_python.payment_class import ResponseCreatePayment
from typing import List
import execjs
from Crypto.PublicKey import RSA
from Crypto.Hash import SHA256
from Crypto.Signature import PKCS1_v1_5
from base64 import b64decode, b64encode
import requests
import re


class MyException(Exception):
    pass


SETI_PRODUCTION = "https://api.seti.dev/v1/"
SETI_DEVELOP = "https://api-ve74dbrnra-uc.a.run.app/"


class Seti:
    def __init__(
        self, private_key: str, api_key: str, public_key="", seti_path=SETI_DEVELOP
    ) -> (None):
        self.private_key = private_key
        self.api_key = api_key
        self.public_key_loc = public_key
        self.seti_path = seti_path

    def verify_sign(self, signature: str, data: dict):
        """
        Verifies with a public key from whom the data came that it was indeed
        signed by their private key
        param: public_key_loc Path to public key
        param: signature String signature to be verified
        return: Boolean. True if the signature is valid; False otherwise.
        """
        if self.public_key_loc == "":
            raise MyException({"msg": "Provide public key path", "code": 1})
        buffered_data = execjs.eval("Buffer.from(JSON.stringify({}))".format(data))
        bytesData = bytearray(buffered_data["data"])
        pub_key = open(self.public_key_loc, "r").read()
        rsakey = RSA.importKey(pub_key)
        signer = PKCS1_v1_5.new(rsakey)
        digest = SHA256.new()
        digest.update(bytesData)
        if signer.verify(digest, b64decode(signature)):
            return True
        return False

    def sign_data(self, data: dict):
        """
        Generate a sign from data with the private key and api_key
        param: private_key Path to private  key
        param: passphrase  api key
        return: Sign data provide
        """
        buffered_data = execjs.eval("Buffer.from(JSON.stringify({}))".format(data))
        bytesData = bytearray(buffered_data["data"])
        rsakey = RSA.importKey(self.private_key)
        hash = SHA256.new(bytesData)
        cipher_rsa = PKCS1_v1_5.new(rsakey)
        return b64encode(cipher_rsa.sign(hash)).decode("utf-8")

    def send_request(self, data: dict, sign_data: str, path: str):
        try:
            url = self.seti_path + path
            r = requests.post(
                url=url,
                json=data,
                headers={"authorization": self.api_key, "sign": sign_data},
                timeout=100,
            )
            return r.json()
        except Exception as e:
            return {"msg": str(e), "code": 5}

    def validate_currency(self, currency: str):
        if not currency in ["COP", "USD", "EUR"]:
            raise MyException({"msg": "Invalid Currency", "code": 2})

    def validate_value(self, value: int):
        if value <= 0:
            raise MyException({"msg": "Invalid value", "code": 3})

    def validate_email(self, email: str):
        regx = r"[^@]+@[^@]+\.[^@]+"
        valid = re.search(regx, email)
        if not valid:
            raise MyException({"msg": "Invalid email format", "code": 4})

    def validate_send_method(self, send_method):
        if not send_method in ["SMS", "WHATSAPP"]:
            raise MyException({"msg": "Invalid Currency", "code": 2})

    def validate_payment_method(self, methods: List):
        if methods.__len__() == 0:
            return
        if len(set(methods) & set(["NEQUI", "PSE", "CARD"])) == 0:
            raise MyException({"msg": "Invalid Payment Method", "code": 4})

    # Payments
    def create_payment(
        self,
        value: int,
        description: str,
        currency: str,
        reference: str,
        redirect_url: str,
        email: str,
        custom_methods: List[str],
    ) -> (ResponseCreatePayment):
        self.is_error = False
        self.validate_currency(currency)
        self.validate_value(value)
        self.validate_payment_method(custom_methods)
        data_create_payment = {
            "value": value,
            "description": description,
            "currency": currency,
            "reference": reference,
            "redirectUrl": redirect_url,
            "email": email,
            "customMethods": custom_methods,
        }
        CREATE_PAYMENT_URL = "create-payment"
        encode_data = self.sign_data(data_create_payment)
        return ResponseCreatePayment(
            self.send_request(data_create_payment, encode_data, CREATE_PAYMENT_URL)
        )

    # Wallets
    def create_wallet(self, email: str, iso_code: str, send_method: str):
        self.is_error = False
        self.validate_email(email)
        self.validate_currency(iso_code)
        self.validate_send_method(send_method)
        data_create_wallet = {
            "email": email,
            "isoCode": iso_code,
            "sendMethod": send_method,
        }
        CREATE_WALLET_URL = "create-wallet"
        encode_data = self.sign_data(data_create_wallet)
        return ResponseCreateWallet(
            self.send_request(data_create_wallet, encode_data, CREATE_WALLET_URL)
        )

    def validate_wallet(self, code: str, secret: str):
        data_validate_wallet = {
            "code": code,
            "secret": secret,
        }
        VALIDATE_WALLET_URL = "validate-wallet"
        encode_data = self.sign_data(data_validate_wallet)
        return CompleteBasicaResponse(
            self.send_request(data_validate_wallet, encode_data, VALIDATE_WALLET_URL)
        )

    def read_wallet_data(self, secret: str):
        self.is_error = False
        data_read_wallet = {
            "secret": secret,
        }
        READ_WALLET_URL = "read-wallet"
        encode_data = self.sign_data(data_read_wallet)
        return ResponseReadWallet(
            self.send_request(data_read_wallet, encode_data, READ_WALLET_URL)
        )

    def read_wallet_transactions(self, secret: str):
        self.is_error = False
        data_read_wallet_transactions = {
            "secret": secret,
        }
        READ_WALLET_TRANSACTIONS_URL = "read-wallet-transactions"
        encode_data = self.sign_data(data_read_wallet_transactions)
        return ResponseReadWalletsTransactions(
            self.send_request(
                data_read_wallet_transactions, encode_data, READ_WALLET_TRANSACTIONS_URL
            )
        )

    def send_money_between_wallets(self, from_secret: str, to_secret: str, value: int):
        self.is_error = False
        self.validate_value(value)
        data_send_money = {
            "fromSecret": from_secret,
            "toSecret": to_secret,
            "value": value,
        }
        SEND_MONEY_WALLETS_URL = "send-money-between-wallets"
        encode_data = self.sign_data(data_send_money)
        return ResponseCreateTransaction(
            self.send_request(data_send_money, encode_data, SEND_MONEY_WALLETS_URL)
        )

    def retention_money(self, from_secret: str, value: int):
        self.is_error = False
        data_retention_money = {"fromSecret": from_secret, "value": value}
        RETENTION_MONEY_URL = "retention-money"
        encode_data = self.sign_data(data_retention_money)
        return ResponseCreateTransaction(
            self.send_request(data_retention_money, encode_data, RETENTION_MONEY_URL)
        )

    # Reversal

    def reversal_payment(self, payment_id: str):
        data_send_money = {"paymentId": payment_id}
        REVERSAL_PAYMENT_URL = "reversal-payment"
        encode_data = self.sign_data(data_send_money)
        return ResponseCreateTransaction(
            self.send_request(data_send_money, encode_data, REVERSAL_PAYMENT_URL)
        )

    # Recharger

    def create_recharger(
        self, value: int, secret: str, custom_methods: List[str], redirect_url: str
    ):
        self.is_error = False
        self.validate_value(value)
        self.validate_payment_method(custom_methods)
        data_create_recharger = {
            "value": value,
            "secret": secret,
            "customMethods": custom_methods,
            "redirectUrl": redirect_url,
        }
        CREATE_RECHARGER_URL = "create-recharger"
        encode_data = self.sign_data(data_create_recharger)
        return ResponseCreatePayment(
            self.send_request(data_create_recharger, encode_data, CREATE_RECHARGER_URL)
        )

    # Dispersion

    def create_dispersion(
        self, value: int, description: str, currency: str, auth_email: str
    ):
        self.is_error = False
        self.validate_value(value)
        data_create_dispersion = {
            "businessEmail": auth_email,
            "value": value,
            "description": description,
            "currency": currency,
        }
        CREATE_DISPERSION_URL = "dispersion-money"
        encode_data = self.sign_data(data_create_dispersion)
        return ResponseCreateDispersions(
            self.send_request(
                data_create_dispersion, encode_data, CREATE_DISPERSION_URL
            )
        )

    def create_dispersion_associated(self, value: int, to_secret: str):
        self.is_error = False
        self.validate_value(value)
        data_create_dispersion = {
            "toSecret": to_secret,
            "value": value,
        }
        CREATE_DISPERSION_URL = "dispersion-money-associated"
        encode_data = self.sign_data(data_create_dispersion)
        return ResponseCreateTransaction(
            self.send_request(
                data_create_dispersion, encode_data, CREATE_DISPERSION_URL
            )
        )

    # Read
    def read_payment(self, payment_id: str):
        self.is_error = False
        data_read_payment = {"paymentId": payment_id}
        READ_PAYMENT_URL = "read-payment"
        encode_data = self.sign_data(data_read_payment)
        return ResponseReadPayment(
            self.send_request(data_read_payment, encode_data, READ_PAYMENT_URL)
        )

    def read_cash(self, currency: str):
        self.is_error = False
        self.validate_currency(currency)
        data_read_cash = {"currency": currency}
        READ_CASH_URL = "read-cash"
        encode_data = self.sign_data(data_read_cash)
        return ResponseReadCash(
            self.send_request(data_read_cash, encode_data, READ_CASH_URL)
        )

    def read_total_cash_outs(self, currency: str):
        self.is_error = False
        self.validate_currency(currency)
        data_read_cash = {"currency": currency}
        READ_TOTAL_CASH_OUT_URL = "read-total-cashout"
        encode_data = self.sign_data(data_read_cash)
        return ResponseReadCashOut(
            self.send_request(data_read_cash, encode_data, READ_TOTAL_CASH_OUT_URL)
        )

    def read_transactions(self):
        self.is_error = False
        data_read_cash = {}
        READ_TRANSACTIONS_URL = "read-transactions"
        encode_data = self.sign_data(data_read_cash)
        return ResponseReadTransactions(
            self.send_request(data_read_cash, encode_data, READ_TRANSACTIONS_URL)
        )

    def create_user(self, email: str, first_name: str, phone: str, iso_code: str):
        self.is_error = False
        self.validate_email(email)
        data_read_cash = {
            "email": email,
            "name": first_name,
            "phone": phone,
            "isoCode": iso_code,
        }
        CREATE_USER_URL = "create-user"
        encode_data = self.sign_data(data_read_cash)
        print(encode_data)
        return ResponseCreateUser(
            self.send_request(data_read_cash, encode_data, CREATE_USER_URL)
        )

    def validate_exist_user(self, email: str):
        self.is_error = False
        self.validate_email(email)
        data_read_cash = {"email": email}
        CREATE_USER_URL = "validate-user"
        encode_data = self.sign_data(data_read_cash)
        return ResponseValidateUsersExist(
            self.send_request(data_read_cash, encode_data, CREATE_USER_URL)
        )

    def create_associated_wallet(self, iso_code: str):
        self.is_error = False
        create_wallet = {"isoCode": iso_code}
        CREATE_ASSOCIATED_WALLET = "create-associated-wallet"
        encode_data = self.sign_data(create_wallet)
        return ResponseCreateWallet(
            self.send_request(create_wallet, encode_data, CREATE_ASSOCIATED_WALLET)
        )


    def revert_transaction(self,transaction_id:str):
        self.is_error = False
        data_read_cash = {"transactionId": transaction_id}
        REVERT_TRANSACTION_URL = "reverse-transaction"
        encode_data = self.sign_data(data_read_cash)
        return CompleteBasicaResponse(
            self.send_request(data_read_cash, encode_data, REVERT_TRANSACTION_URL)
        )



class BasicResponse:
    msg: str
    res: int

    def __repr__(self):
        return str(self.__dict__)


class CompleteBasicaResponse(BasicResponse):

    def __init__(self, data: dict):
        self.msg = data['msg']
        self.res = data['res']
        if data['res'] == 200:
            self.data = ResponseCreateTransactionData(data['data'])
        else:
            self.data = ResponseCreateTransactionData({})


class ResponseCreateTransactionData:
    transaction_id: str

    def __init__(self, data: dict):
        try:
            self.transaction_id = data['transactionId']
        except Exception:
            return


class ResponseCreateTransaction(BasicResponse):
    data: ResponseCreateTransactionData

    def __init__(self, data: dict):
        self.msg = data['msg']
        self.res = data['res']
        if data['res'] == 200:
            self.data = ResponseCreateTransactionData(data['data'])
        else:
            self.data = ResponseCreateTransactionData({})


class ResponseCreateDispersionsData:
    ip: str
    reference: str
    timestamp: dict
    transactionType: str
    value: int

    def __init__(self, data: dict):
        try:
            self.ip = data['ip']
            self.reference = data["reference"]
            self.timestamp = data["timestamp"]
            self.transactionType = data["transactionType"]
            self.value = data["value"]
        except Exception:
            return


class ResponseCreateDispersions(BasicResponse):
    data: ResponseCreateDispersionsData

    def __init__(self, data: dict):
        self.msg = data['msg']
        self.res = data['res']
        if data['res'] == 200:
            self.data = ResponseCreateDispersionsData(data['data'])
        else:
            self.data = ResponseCreateDispersionsData({})



from seti_python.class_tools import BasicResponse


class ResponseCreatePaymentData:
    id: str
    url: str

    def __init__(self, data: dict):
        try:
            self.id = data["id"]
            self.url = data["url"]
        except Exception:
            return


class ResponseCreatePayment(BasicResponse):
    data: ResponseCreatePaymentData

    def __init__(self, data: dict):
        print("----------------------------------")
        print(data)
        self.msg = data["msg"]
        self.res = data["res"]
        if data["res"] == 200:
            self.data = ResponseCreatePaymentData(data["data"])
        else:
            self.data = ResponseCreatePaymentData({})




from seti_python.class_tools import BasicResponse


class ResponseCreateWalletData:
    id: str
    url: str

    def __init__(self, data: dict):
        try:
            self.secret = data['secret']
        except Exception:
            return


class ResponseCreateWallet(BasicResponse):
    data: ResponseCreateWalletData

    def __init__(self, data: dict):
        self.msg = data['msg']
        self.res = data['res']
        if data['res'] == 200:
            self.data = ResponseCreateWalletData(data['data'])
        else:
            self.data = ResponseCreateWalletData()




from seti_python.class_tools import BasicResponse


# Read total CashOUts
class ValidateUsersExist:
    exist = False

    def __init__(self, data: dict):
        self.exist = data['exist']


class ResponseValidateUsersExist(BasicResponse):
    data: ValidateUsersExist

    def __init__(self, data: dict):
        self.msg = data['msg']
        self.res = data['res']
        if data['res'] == 200:
            self.data = ValidateUsersExist(data['data'])
        else:
            self.data = ValidateUsersExist({})

# Create user Data


class NewUsers:
    secret: str

    def __init__(self, data: dict):
        try:
            self.secret = data['secret']
        except Exception:
            self.secret = ""


class ResponseCreateUser(BasicResponse):
    data: NewUsers

    def __init__(self, data: dict):
        self.msg = data['msg']
        self.res = data['res']
        if data['res'] == 200:
            self.data = NewUsers(data['data'])
        else:
            self.data = NewUsers({})



from distutils.core import setup
import setuptools


setup(
    name='seti-python',
    version='0.1.9',
    include_package_data=True,
    description='Library to use conect to seti functions',
    author='Andres Gonzalez',
    author_email='andres@4coders.co',
    license="GPLv3",
    # use the URL to the github repo
    url='https://github.com/4CodersColombia/seti-python',
    download_url='https://github.com/4CodersColombia/seti-python/archive/refs/tags/0.0.2.tar.gz',
    keywords=['python'],
    classifiers=['Programming Language :: Python :: 3.9', ],
    packages=setuptools.find_packages(),
    install_requires=[
        'PyExecJS==1.5.1',
        'pycryptodome==3.10.1',
    ],
)
